/*
Copyright © 2011 Marco Mescoli, Erio Orlandi, Franco Melchionna,
Andrea Gavioli, Federico Carnevali, Mario Benito Totaro

This file is part of roboino project.
roboino is a little educational robot with free firmware:
you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
roboino project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with roboino project. If not, see <http://www.gnu.org/licenses/>.
*/

#include "robo31.h"

volatile int stepsSx = 0; 
volatile int stepsDx = 0; 
volatile int print = 0; 

unsigned long _ItIni = 0;   // start time
char strBuff[PbUFFER];
byte opto;

// 2016-12-18
// test software che integra tutte le nuove funzionalita'
// il mio roboino 3.0 ha problema al fotodiodo della fotocoppia centrale
// perche' e' alimentato a spot (provato con programmi specifici)
// e la fotocoppia cx in lettura legge 1 costante per cui delle combinazioni 
// sotto con questo main sul mio roboino sono possibili solo la 2,3(vuota),6,7

void loop( void ){
  if (analogRead(BUTTON) > 800 ) test();
  else { 
    opto=optoRead(4,10);
    if ( opto == 0 ) fido( false );       // 2016-12-16 collaudo ok
    if ( opto == 1 ) test();              // 2016-12-16 collaudo ok
    if ( opto == 2 ) vigile();            // non riesco a provarlo ....
    if ( opto == 5 ) segugio( false );    // 2016-12-16 collaudo ok pur mancando fotocoppia cx
    if ( opto == 6 ) tank();              // 2016-12-16 funzia con problemini di orientamento schermo e di btConnect
    if ( opto == 7 ) blueToothDrive();    // 2016-12-16 funzia
  }
}

void test( void ){
  Serial.print( "liveAs: continous tests.\r\n" );
  for (;;) {
    testLeds();
    dcMotorConsole();
    testProxOpto();
    testAnalog();
    testEncoders();
  }
}
void dcMotorConsole( void ){
  int sx = 0, dx = 0;
  char key;
  bool esc = false;
  unsigned long start, duration = 60000;
   
  Serial.print( "1 minutes test motors. Enter keypad numbers to run motors. '0' to exit '*' to continue forever.\r\n" );
  dcMotorPWM( 0, 0, true );
  start = millis();
  do {
    key = 0;
    if ( Serial.available() ) while ( Serial.available() > 0 ) key = Serial.read();
    if ((( key == '0' ) || ( millis() >= ( start +  duration ))) && key != '*') esc = true;
    if ( key ) {
      switch ( key ) {
        case '7': sx+=5; break;
        case '4': sx = 0; break;
        case '1': sx-=5; break;
        case '8': sx+=5; dx+=5; break;
        case '5': sx = 0; dx = 0; break;
        case '2': sx-=5; dx-=5; break;
        case '9': dx+=5; break;
        case '6': dx = 0; break;
        case '3': dx-=5; break;
        case '0': esc = true; break;
      }
      sx = constrain( sx, MINd, MAXd );
      dx = constrain( dx, MINd, MAXd );
      dcMotorPWM( sx, dx, true );
    }
  } while ( !esc );
  dcMotorPWM( 0, 0, true );
}

void testProxOpto( void ) {
  unsigned long start, duration = 60000; 
  unsigned int pxSx, pxDx;
  char opto;
  bool esc = false;
  char key=0;

  Serial.print( "1 minute test: prox, opto read. '0' or 'esc' to exit '*' to continue forever.\r\n" );
  start = millis();
  do {
    if ( Serial.available() ) key = Serial.read();
    if ((( key == '0' ) || ( millis() >= ( start +  duration ))) && key != '*') esc = true;
    pxSx = distSx( true );
    pxDx = distDx( true );
    if ( pxSx !=0 && pxDx != 0 ) {
      opto = optoRead( 4, 0 );
      sprintf( strBuff, "pxSx:%6d    pxDx:%6d    opto:%2d\n\r", pxSx, pxDx, opto );
      Serial.print( strBuff );
    }
  } while( !esc );
}

void testLeds( void ){
  char key=0;
  unsigned long start, duration = 3000; 
  bool esc = false;

  Serial.print( "3 secs test white led. '0' or 'esc' to exit '*' to continue forever.\r\n" );
  start = millis();
  do {
    if ( Serial.available() ) key = Serial.read();
    if ((( key == '0' ) || ( millis() >= ( start +  duration ))) && key != '*') esc = true;
    digitalWrite( LEDeXT, HIGH);
    digitalWrite( LEDiNT, LOW);
    delay( 500 );
    digitalWrite( LEDeXT, LOW);
    digitalWrite( LEDiNT, HIGH);
    delay( 500 );
    Serial.print("leds\r\n");
  } while( !esc );
  digitalWrite( LEDiNT, LOW);
  digitalWrite( LEDeXT, LOW);
}

void testAnalog( void ){
  char key=0;
  unsigned long start, duration=10000;
  bool esc = false;
  int button, analogIn;
  
  Serial.print( "10 seconds test button and analog in. '0' or 'esc' to exit '*' to continue.\r\n" );
  start = millis();
  do {
    if ( Serial.available() ) key = Serial.read();
    if ((( key == '0' ) || ( millis() >= ( start +  duration ))) && key != '*') esc = true;
    button = analogRead( BUTTON );
    analogIn = analogRead( ANALOGiN );
    delay( 100 );
    sprintf( strBuff, "button:%6d    analogIn:%6d\n\r", button, analogIn );
    Serial.print( strBuff );
  } while( !esc );
}

// test with interrupts
void testEncoders( void ){
  char key=0;
  unsigned long start, duration = 60000;
  bool esc = false;
  
  Serial.print( "60 seconds encoders. '0' or 'esc' to exit '*' to continue.\r\n" );
  start = millis();
  do {
    if ( Serial.available() ) key = Serial.read();
    if ((( key == '0' ) || ( millis() >= ( start +  duration ))) && key != '*') esc = true;
    if ( analogRead( BUTTON ) > 800 ) { stepsSx = 0; stepsDx = 0; }
    if ( print ) {
      Serial.print( "sx:" );
      Serial.print( stepsSx );
      Serial.print( "   dx:" );
      Serial.println( stepsDx );
      print = 0;
    }
    digitalWrite( LEDeXT, digitalRead(ENCODERsX1) );
    digitalWrite( LEDeXT, digitalRead(ENCODERdX1) );
  } while( !esc );
}

void brrr( int times, int msPause ) {
  int n;
  for ( n = 0; n < times; n++ ){
    digitalWrite( LEDeXT, HIGH );
    dcMotorPWM( MAXd, MINd, false );
    delay( msPause );
    digitalWrite( LEDeXT, LOW );
    dcMotorPWM( MINd, MAXd, false );
    delay( msPause );
  }
  dcMotorPWM( 0, 0, false );
  delay( 100 );
}

void fido( bool debug ){
  float m1, m2, q1, q2;
  float pxSx, pxDx;
  int sx, dx;
  //int opto;
  //unsigned long startTime;
  //float seconds;

  Serial.print( "liveAs: fido follow or escape from you\r\n" );
  // each sensor controls its wheel
  // under ZERO rest
  // between ZERO and NEAR speed = m1 * px + q1 
  // between NEAR and FAR rest 
  // between FAR and REMOTE speed = m2* px + q2
  // aover REMOTE rest
  m1 = MINf / ( ZERO - NEAR );
  q1 = -m1 * NEAR;
  m2 = MAXf / ( REMOTE - FAR );
  q2 = - m2 * FAR;
  Serial.println("start ...");
  //startTime = millis();
  pxSx = pxDx = 0;
  while( true ){
    pxSx = (float) distSx( false );
    sx = 0;
    if ( pxSx < FARrEMOTE && pxDx < FARrEMOTE ) {
      if ( pxSx >= ZERO && pxSx <= NEAR ) sx = -10 + (int)( m1 * pxSx + q1 ); // -+ go back
      if ( pxSx >= FAR && pxSx <= REMOTE ) sx = 10 + (int)( m2 * pxSx + q2 ); //++ go forward
    }
    delay( 30 );
    pxDx = (float) distDx( false );
    dx = 0; 
    if ( pxSx < FARrEMOTE && pxDx < FARrEMOTE ) {
      if ( pxDx >= ZERO && pxDx <= NEAR ) dx = -10 + (int)( m1 * pxDx + q1 ); // -+ go back
      if ( pxDx >= FAR && pxDx <= REMOTE ) dx = 10 + (int)( m2 * pxDx + q2 ); //++ go forward
    }
    if (( pxSx >= REMOTE && pxSx <= FARrEMOTE ) && ( pxDx >= REMOTE && pxDx <= FARrEMOTE )) {
      brrr( 2, 100 );
      delay( 50 );
    }
    /*
    if ( dx || sx ) {
      sprintf( strBuff, "sx=%3d dx=%3d pxSx=%3d pxDx=%3d\n\r", sx, dx, (int)pxSx, (int)pxDx );
      Serial.print( strBuff );
    }
    */
    dcMotorPWM( sx, dx, false );
    /*
    opto = optoRead( 30, 10 );
    if ( opto == 6 || opto == 3 ) {
      beeBoop(2,1,50);
      seconds = (millis()-startTime)/1000.0;
      Serial.print(seconds);
      Serial.println(" sec error !");
      brrr( 2, 100 );
      delay(200);
      brrr( 2, 100 );
      delay(200);
    }
    if ( opto == 5 ) {
      seconds = (millis()-startTime)/1000.0;
      Serial.print(seconds);
      Serial.println(" sec :-)");
      Serial.println("stop.");
      dcMotorPWM( 0, 0, false );
      beeBoop( 1, 100, 0 );
      delay(200);
      beeBoop( 1, 100, 0 );
      delay(200);
      beeBoop( 1, 100, 0 );
      delay(200);
      for(;;);
    }
    */
    if (debug) {
      sprintf( strBuff, "sx=%3d dx=%3d pxSx=%3d pxDx=%3d\n\r", sx, dx, (int)pxSx, (int)pxDx );
      Serial.print( strBuff );
    }
  }
}

void segugio( bool debug ){
  char speedSx, speedDx;
  byte read;
  int pista[8][2] = {          
    {0,0},  
    {MAXm/2, 0}, 
    {MAXm, MAXm}, 
    {MAXm/2, MINm/2},  //oppa          
    {0, MAXm/2},            
    {MAXm, MAXm},          
    {MINm/2, MAXm/2},         
    {MINm, MINm}
  };
   
  Serial.print( "liveAs: Segugio follow and sniff for black track\r\n" );
  do {
    read = optoRead( 4, 0);
    read = (~read) & 0b00000111;
    if ( debug ) {
      Serial.println( read );
      Serial.flush();
    }
    speedDx = pista[read][0];
    speedSx = pista[read][1];    
    dcMotorPWM( speedSx, speedDx, false );
  } while ( true );
  dcMotorPWM( 0, 0, false );
}
