/*
Copyright © 2011 Marco Mescoli, Erio Orlandi, Franco Melchionna,
Andrea Gavioli, Federico Carnevali, Mario Benito Totaro

This file is part of roboino project.
roboino is a little educational robot with free firmware:
you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
roboino project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with roboino project. If not, see <http://www.gnu.org/licenses/>.
*/

void setup( void ) {
//------ dc motors ----------
  pinMode( DCdXCLK, OUTPUT );
  pinMode( DCdXACLK, OUTPUT );
  pinMode( DCsXCLK, OUTPUT );
  pinMode( DCsXACLK, OUTPUT );

//------ outSignals ----------
  pinMode( LEDeXT, OUTPUT );
  digitalWrite( LEDeXT, LOW );
  pinMode( BUZZER, OUTPUT );

//------ inSignals ----------
  pinMode( BUTTON, INPUT );
  pinMode( ANALOGiN, INPUT );

//------ sonars ----------
  pinMode( TRIGGERsX, OUTPUT );
  pinMode( ECHOsX, INPUT );
  pinMode( TRIGGERdX, OUTPUT );
  pinMode( ECHOdX, INPUT );
  digitalWrite( TRIGGERsX, LOW );
  digitalWrite( TRIGGERdX, LOW );

//------ opto pairs ----------
  pinMode( OPTOsX, INPUT );
  pinMode( OPTOcX, INPUT );
  pinMode( OPTOdX, INPUT );
  pinMode( OPTOcXtX, OUTPUT );
  digitalWrite(OPTOcXtX, HIGH);

//------ encoders ---
  pinMode( ENCODERsX0, INPUT );
  pinMode( ENCODERsX1, INPUT );
  pinMode( ENCODERdX0, INPUT );
  pinMode( ENCODERdX1, INPUT );
  attachInterrupt( 0, countSx, RISING );
  attachInterrupt( 1, countDx, RISING );

  // --- i am rebooting ------
  beeBoop(3,10,100);

  digitalWrite(OPTOcXtX,HIGH);
  Serial.begin( 9600 );   
  delay( 100 );   // wait transitories end
  Serial.print( ALIVE );
  dcMotorPWM( 0, 0, false );
}

// =================== showel opto couples ==================
char optoRead( int repeat, int msPause ) {
  byte sx, cx, dx;
  char read, prevRead;
  int n = 0;

  digitalWrite(OPTOcXtX,HIGH);
  sx = digitalRead(OPTOsX);             // bit 0 left
  cx = digitalRead(OPTOcX) << 1;        // bit 1 central
  dx = digitalRead(OPTOdX) << 2;        // bit 2 right
  read = (sx | cx | dx) & 0b00000111;
  do {
    prevRead = read;
    sx = digitalRead(OPTOsX);           // bit 0 left
    cx = digitalRead(OPTOcX) << 1;    // bit 1 central
    dx = digitalRead(OPTOdX) << 2;            // bit 2 right
    read = (sx | cx | dx) & 0b00000111;       // true => black, false => white
    if ( read != prevRead && repeat > 1 ) break;
    if ( msPause > 0 ) delay( msPause );
    n++;
  } while ( n < repeat && read != -1);
  digitalWrite(OPTOcXtX,LOW);
  return read;
}

// -50 .. 50 each motor
void dcMotorPWM( int sx, int dx, bool debug ){
  // -------------- LEFT dcMotor ----------------- 
  sx = constrain( sx, MINd, MAXd );
  if (sx > 0) {
    analogWrite( DCsXCLK, 0 );
    analogWrite( DCsXACLK, map( sx, 0, MAXd, 0, 255 ) );
  }
  else {
    analogWrite( DCsXCLK, map( sx, MINd, 0, 255, 0 ) );
    analogWrite( DCsXACLK, 0 );
  }
  // -------------- RIGHT dcMotor ----------------- 
  dx = constrain( dx, MINd, MAXd );
  if ( dx > 0) {
    analogWrite( DCdXCLK, map( dx, 0, MAXd, 0, 255 ) ); 
    analogWrite( DCdXACLK, 0 );
  }
  else {
    analogWrite( DCdXCLK, 0 );
    analogWrite( DCdXACLK, map( dx, MINd, 0, 255, 0 ) );
  }
  // -------------- debug ----------------- 
  if ( debug ) {
    sprintf( strBuff, " sx:%4d  dx:%4d\n\r", sx, dx );
    Serial.print( strBuff );
  }
}

// ============== encoders ========================
void countSx() {
  if (digitalRead(ENCODERsX1)) stepsSx++;
  else stepsSx--;
  print = 1;
}

void countDx() {
  if (digitalRead(ENCODERdX1)) stepsDx++;
  else stepsDx--;
  print = 1;
}

// ============== sonar ========================
// the sonars must not interact
// subsequent acquisition left right
// must 10 ms paused
// mmDist = 0.34 * dt / 2.0;
// -------------- left sonar --------------
unsigned int distSx( bool pause ) {  
  if ( pause ) delay( 10 );
  digitalWrite( TRIGGERsX, HIGH );
  delayMicroseconds( 20 );
  digitalWrite( TRIGGERsX, LOW );
  return pulseIn( ECHOsX, HIGH, SONARtIMEoUT ); // us
}

// -------------- right sonar --------------
unsigned int distDx( bool pause ) { 
  if ( pause ) delay( 10 );
  digitalWrite( TRIGGERdX, HIGH );
  delayMicroseconds( 20 );
  digitalWrite( TRIGGERdX, LOW );
  return pulseIn( ECHOdX, HIGH, SONARtIMEoUT ); // us
}

// ------ buzzer --------
// msOn = 1 msOff = 1..100
// 1:1 alarm, 1:100 sleep ... 
void beeBoop( unsigned int times, unsigned int msOn, unsigned int msOff ){
  for ( unsigned int i=0; i<times; i++ ) {
    digitalWrite( BUZZER, HIGH );
    delay( msOn );
    digitalWrite( BUZZER, LOW );
    delay( msOff );
  }
}
