/*
Copyright © 2011 Marco Mescoli, Erio Orlandi, Franco Melchionna,
Andrea Gavioli, Federico Carnevali, Mario Benito Totaro

This file is part of roboino project.
roboino is a little educational robot with free firmware:
you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
roboino project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with roboino project. If not, see <http://www.gnu.org/licenses/>.
*/

void blueToothDrive( void ){
// ----------------- input ---------------------
  int gas = 0;              // sensoduino gas -180 +180, pitch -180..+180 (0 telefono sul piano -90 in piedi + 90 testa in giu')
  int wheel = 0;            // sensoduino wheel -180..+180 (0 cell sul piano -90 appoggiato sul lato lungo guarda a dx + 90 guarda a sx) 
  bool joystick;            // smartPos used as dish or joystick
// ----------------- status ---------------------
  int noBt = 0;             // times bluetooth non available
  int tooMuch = 0;          // times too much gas or speed
// ----------------- ouput ---------------------
  int sx = 0;               // speed left DCMotor -50..50
  int dx = 0;               // speed right DCMotor --50..50
  int err;

  Serial.print( "liveAs: blueTooth joystick\r\n" );
  for(;;) {
    err = getBtJoystick( &gas, &wheel, &noBt, joystick, false );
    if ( err == BTdOWN ) { 
      dcMotorPWM( 0, 0, false );
      joystick = false;
    }
    if ( !err ) {
      computeSpeed( gas, wheel, &sx, &dx, &tooMuch, false );
      dcMotorPWM( sx, dx, false );
    }
  }
}

void computeSpeed( int gas, int wheel, int *speedSx, int * speedDx, int *tooMuch, bool debug ){
  int lGas, lWheel;

  if (( abs( gas ) > 110 ) || ( abs( wheel ) > 80 )) {
    (*tooMuch)++;
    if ( *tooMuch >= 2 ) {
      *tooMuch = 0;
      brrr(10,100);
    }
  }
  else {
    *tooMuch = 0;
    lGas = constrain( gas, MINd, MAXd );
    if ( abs( lGas ) < 7 ) lGas = 0;
    else lGas += 15 * abs( lGas ) / lGas;
    lWheel = constrain( wheel, MINd, MAXd );
    if ( abs( lWheel ) < 7 ) lWheel = 0;
    else  lWheel += 5 * abs( lWheel ) / lWheel;
    *speedSx = lGas - lWheel;
    *speedDx = lGas + lWheel;
  }
  if ( debug ) {
    sprintf( strBuff, " sx:%4d  dx:%4d\n\r", *speedSx, *speedDx  );
    Serial.print( strBuff );
  }
}

int getBtJoystick( int *gas, int *wheel, int *noBt, bool joystick, bool debug ) {
/*
  if (Serial.available() < 1) {
    (*noBt)++;    // if don't put parenthesis operator ++ act on pointer
    if ( *noBt >= BTmAXdOWN ) { // n times sequence
      Serial.print("noBluetooth joystick\r\n");
      *noBt = 0;  
      return BTdOWN;
    }
    return BToNEeRR; // Serial not available
  }
*/
  char getChar = Serial.read();
  *gas = 0;
  if ( joystick ) *gas = 90;
  *wheel = 0;
  if (getChar != STARTcMDcHAR) {
    *noBt = 0;
    return BTcHAReRR;
  }
  // there's incoming data: parsing  
  // sensorType 3 => orientation
  if ( Serial.parseInt() == ORIENTATION ) {
    Serial.parseInt();     // log count: not needed
    Serial.parseFloat();   // north: not needed
    *noBt = 0;
    *gas   = (int) Serial.parseFloat(); // sensoduino gas -180 +180 as dish, -90 +270 as joystick
    if ( joystick ) (*gas) += 90;
    *wheel = (int) Serial.parseFloat(); // sensoduino wheel -180 +180
    if ( debug ) {
      sprintf( strBuff, "gas: %4d  wheel: %4d\n\r", *gas, *wheel );
      Serial.print( strBuff );
    }
    return 0;
  }
  else return BTwRONGsENS; // acquisition done but wrong sensor
}

